@@ -1,8 +1,8 @@
 ######################################################################
 # stepwiseqtl.R
 #
-# copyright (c) 2007-2015, Karl W Broman
-# last modified Jun, 2015
+# copyright (c) 2007-2021, Karl W Broman
+# last modified Mar, 2021
 # first written Nov, 2007
 #
 #     This program is free software; you can redistribute it and/or
@@ -62,7 +62,7 @@ stepwiseqtl <-
              keeplodprofile=TRUE, keeptrace=FALSE, verbose=TRUE,
              tol=1e-4, maxit=1000, require.fullrank=FALSE)
 {
-    if(!("cross" %in% class(cross)))
+    if(!inherits(cross, "cross"))
         stop("Input should have class \"cross\".")
 
     if(!missing(chr)) cross <- subset(cross, chr)
@@ -85,8 +85,8 @@ stepwiseqtl <-
             warning("penalties should be a vector; only the first row will be used")
         }
         if(length(penalties)==6) { # X-chr-specific penalties
-            chrtype <- vapply(cross$geno, class, "")
-            if(!all(chrtype=="A")) {
+            chr_type <- vapply(cross$geno, chrtype, "")
+            if(!all(chr_type=="A")) {
                 if(scan.pairs)
                     warning("scan.pairs=TRUE not implemented X-chr specific penalties; ignored.")
                 return(stepwiseqtlX(cross, chrnames(cross), pheno.col=pheno.col, qtl=qtl,
@@ -105,16 +105,16 @@ stepwiseqtl <-
         pheno.col <- 1
     }
 
-    chrtype <- sapply(cross$geno, class)
-    if(any(chrtype=="X")) {
-        Xadjustment <- scanoneXnull(class(cross)[1], getsex(cross), attributes(cross))
+    chr_type <- sapply(cross$geno, chrtype)
+    if(any(chr_type=="X")) {
+        Xadjustment <- scanoneXnull(crosstype(cross), getsex(cross), attributes(cross))
         forceXcovar <- Xadjustment$adjustX
         Xcovar <- Xadjustment$sexpgmcovar
     }
     else forceXcovar <- FALSE
 
     if(!is.null(qtl)) { # start f.s. at somewhere other than the null
-        if( !("qtl" %in% class(qtl)) )
+        if( !inherits(qtl, "qtl") )
             stop("The qtl argument must be an object of class \"qtl\".")
 
         # check that chromosomes were retained, otherwise give error
@@ -215,13 +215,14 @@ stepwiseqtl <-
     pheno <- cross$pheno[,pheno.col]
     if(!is.null(covar)) phcovar <- cbind(pheno, covar)
     else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
-    hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
+    hasmissing <- rowSums(is.na(phcovar)) > 0
     if(all(hasmissing))
         stop("All individuals are missing phenotypes or covariates.")
     if(any(hasmissing)) {
         pheno <- pheno[!hasmissing]
         cross <- subset(cross, ind=!hasmissing)
         if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]
+        if(forceXcovar) Xcovar <- Xcovar[!hasmissing,,drop=FALSE]
 
         if(!startatnull) {
             if(method=="imp")
@@ -244,7 +245,12 @@ stepwiseqtl <-
         else firstformula <- formula
     }
     else {
-        lod0 <- length(pheno)/2 * log10(sum((pheno-mean(pheno))^2) / sum(lm(pheno ~ as.matrix(covar))$resid^2))
+        nullformula <- as.formula(paste("y~", paste(names(covar), collapse="+")))
+        tempqtl <- makeqtl(cross, chrnames(cross)[1], 0, what=ifelse(method=="imp", "draws", "prob"))
+        fit <- fitqtl(cross, pheno.col, tempqtl, covar=covar, formula=nullformula,
+                      method=method, model=model, dropone=FALSE, get.ests=FALSE,
+                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
+        lod0 <- fit$result.full[1,4]
         if(startatnull)
             firstformula <- as.formula(paste("y~", paste(names(covar), collapse="+"), "+", "Q1"))
         else firstformula <- formula
@@ -252,7 +258,7 @@ stepwiseqtl <-
 
 
     # penalties
-    cross.type <- class(cross)[1]
+    cross.type <- crosstype(cross)
     if(missing(penalties)) {
         if(cross.type=="f2") {
             penalties <-  c(3.52, 4.28, 2.69)
@@ -365,6 +371,7 @@ stepwiseqtl <-
             if(verbose) cat(" ---Refining positions\n")
             rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                               covar=covar, formula=formula, method=method,
+                              model=model,
                               verbose=verbose.scan, incl.markers=incl.markers,
                               keeplodprofile=FALSE, forceXcovar=forceXcovar)
             if(any(rqtl$pos != qtl$pos)) { # updated positions
@@ -376,6 +383,7 @@ stepwiseqtl <-
                       method=method, model=model, dropone=FALSE, get.ests=FALSE,
                       run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
         lod <- fit$result.full[1,4] - lod0
+
         if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0
         curplod <- calc.plod(lod, countqtlterms(formula, ignore.covar=TRUE),
                              penalties=penalties)
@@ -419,7 +427,7 @@ stepwiseqtl <-
         }
 
         out <- addqtl(cross, pheno.col=pheno.col, qtl=qtl, covar=covar,
-                      formula=formula, method=method, incl.markers=incl.markers,
+                      formula=formula, method=method, model=model, incl.markers=incl.markers,
                       verbose=verbose.scan, forceXcovar=forceXcovar,
                       require.fullrank=require.fullrank)
 
@@ -445,7 +453,7 @@ stepwiseqtl <-
                 thisformula <- as.formula(paste(deparseQTLformula(formula), "+Q", n.qtl+1,
                                                 "+Q", j, ":Q", n.qtl+1, sep=""))
                 out <- addqtl(cross, pheno.col=pheno.col, qtl=qtl, covar=covar,
-                              formula=thisformula, method=method, incl.markers=incl.markers,
+                              formula=thisformula, method=method, model=model, incl.markers=incl.markers,
                               verbose=verbose.scan, forceXcovar=forceXcovar,
                               require.fullrank=require.fullrank)
                 thislod <- max(out[,3], na.rm=TRUE)
@@ -474,7 +482,7 @@ stepwiseqtl <-
                 if(verbose)
                     cat(" ---Look for additional interactions\n")
                 temp <- addint(cross, pheno.col, qtl, covar=covar, formula=formula,
-                               method=method, qtl.only=TRUE, verbose=verbose.scan,
+                               method=method, model=model, qtl.only=TRUE, verbose=verbose.scan,
                                require.fullrank=require.fullrank)
                 if(!is.null(temp)) {
                     thislod <- max(temp[,3], na.rm=TRUE)
@@ -499,7 +507,7 @@ stepwiseqtl <-
                 if(verbose)
                     cat(" ---Scan for an additional pair\n")
                 out <- addpair(cross, pheno.col=pheno.col, qtl=qtl, covar=covar,
-                               formula=formula, method=method, incl.markers=incl.markers,
+                               formula=formula, method=method, model=model, incl.markers=incl.markers,
                                verbose=verbose.scan, forceXcovar=forceXcovar)
                 thelod <- out$lod
 
@@ -555,7 +563,7 @@ stepwiseqtl <-
         if(refine.locations) {
             if(verbose) cat(" ---Refining positions\n")
             rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
-                              covar=covar, formula=formula, method=method,
+                              covar=covar, formula=formula, method=method, model=model,
                               verbose=verbose.scan, incl.markers=incl.markers,
                               keeplodprofile=FALSE, forceXcovar=forceXcovar)
             if(any(rqtl$pos != qtl$pos)) { # updated positions
@@ -687,7 +695,7 @@ stepwiseqtl <-
             if(verbose) cat(" ---Refining positions\n")
             if(!is.null(qtl)) {
                 rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
-                                  covar=covar, formula=formula, method=method,
+                                  covar=covar, formula=formula, method=method, model=model,
                                   verbose=verbose.scan, incl.markers=incl.markers,
                                   keeplodprofile=FALSE, forceXcovar=forceXcovar)
                 if(any(rqtl$pos != qtl$pos)) { # updated positions
@@ -747,7 +755,7 @@ stepwiseqtl <-
         if(keeplodprofile) {
             if(verbose) cat(" ---One last pass through refineqtl\n")
             qtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
-                             covar=covar, formula=formula, method=method,
+                             covar=covar, formula=formula, method=method, model=model,
                              verbose=verbose.scan, incl.markers=incl.markers,
                              keeplodprofile=TRUE, forceXcovar=forceXcovar)
         }
@@ -930,7 +938,7 @@ countqtlterms <-
 calc.penalties <-
     function(perms, alpha=0.05, lodcolumn)
 {
-    if(missing(perms) || !("scantwoperm" %in% class(perms)))
+    if(missing(perms) || !inherits(perms, "scantwoperm"))
         stop("You must include permutation results from scantwo.")
 
     if("AA" %in% names(perms)) { # X-chr-specific penalties
